﻿package ld16
{
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import ld16.gamestate.Overworld;
    import ld16.subgame.gamestate.SuperEasy;
    import ld16.subgame.SubGame;
    import ld16.subgame.TestGameState;
    import stego.constants.Colors;
    import stego.core.BaseGame;
    import stego.core.GameObject;
    import stego.core.GameState;
    import stego.util.SceneLayers;
    import stego.widget.PulsingTextButton;
    import stego.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class OverworldScreen extends GameObject
    {
        public static const EVENT_PLAYBUTTON_PRESSED:String = "play button pressed";
        public static const EVENT_QUITBUTTON_PRESSED:String = "quit button pressed";

        private static const TOP_MARGIN:Number    = 80;
        private static const BOTTOM_MARGIN:Number = 120;
        private static const SIDE_MARGIN:Number   = 100;
        private static const TRANSITION_TIME:Number = 0.25;

        public function get title():String { return _title; }
        public function set title(value:String):void
        {
            _title = value;
            if (null != titleTextfield)
            {
                titleTextfield.SetText(value);
            }
        }
        private var _title:String = "<<<TITLE>>>";

        public function get hint():String { return _hint; }
        public function set hint(value:String):void { _hint = value; }
        private var _hint:String = "<<<hint>>>";

        public function get gameState():GameState { return _gameState; }
        public function set gameState(value:GameState):void
        {
            if (null != _gameState)
            {
                _gameState.removeEventListener(Overworld.EVENT_WON, WonMiniGame);
                _gameState.removeEventListener(Overworld.EVENT_LOST, LostMiniGame);
            }

            _gameState = value;
            if (null != _gameState)
            {
                _gameState.addEventListener(Overworld.EVENT_WON, WonMiniGame);
                _gameState.addEventListener(Overworld.EVENT_LOST, LostMiniGame);
            }
        }
        private var _gameState:GameState;

        private function WonMiniGame(e:Event):void
        {
            dispatchEvent(new Event(Overworld.EVENT_WON));
            scheduler.CallOnce(0.7, QuitButtonPressed);
        }
        private function LostMiniGame(e:Event):void
        {
            dispatchEvent(new Event(Overworld.EVENT_LOST));
            scheduler.CallOnce(1.0, QuitButtonPressed);
        }

        private var game:BaseGame = new SubGame();
        private var background:Sprite;
        private var playButton:PulsingTextButton;
        private var quitButton:PulsingTextButton;
        private var titleTextfield:TextField;
        private var gameMask:Sprite;

        public function OverworldScreen()
        {
            gameState = new TestGameState();
        }

        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            PrepareGame();

            // start invisible
            visible = false;
        }

        /**
         * make sure the game is all hooked up and ready to be started
         */
        private function PrepareGame():void
        {
            if (null != game)
            {
                addChild(game);
                game.mask = gameMask;
                game.x = G.SubGameRect.left;
                game.y = G.SubGameRect.top;
            }
        }

        public function CreateGraphics():void
        {
            if (null == background)
            {
                background = new Sprite();
                background.graphics.lineStyle(2, Colors.WHITE);
                background.graphics.beginFill(Colors.BLACK);
                background.graphics.drawRect(
                    G.SubGameRect.left,
                    G.SubGameRect.top,
                    G.SubGameRect.width,
                    G.SubGameRect.height);
                background.graphics.endFill();
                addChild(background);
            }

            if (null == titleTextfield)
            {
                titleTextfield = InternalGameObjects.Add(TextField) as TextField;
                titleTextfield.SetFontColor(Colors.WHITE_ENOUGH);
                titleTextfield.SetFont(GameContent.TuffyBoldFont);
                titleTextfield.SetFontSize(72);
                titleTextfield.SetText(title);
                titleTextfield.CenterOn( G.SubGameRect.left + (0.5 * (G.SubGameRect.right - G.SubGameRect.left)),
                                         G.SubGameRect.top  + (0.4 * (G.SubGameRect.bottom - G.SubGameRect.top) ) );
            }

            if (null == playButton)
            {
                playButton = InternalGameObjects.Add(PulsingTextButton) as PulsingTextButton;
                playButton.SetFontColor(Colors.WHITE_ENOUGH);
                playButton.SetHoverColor(Colors.WHITE);
                playButton.SetFont(GameContent.TuffyBoldFont);
                playButton.SetFontSize(48);
                playButton.SetText("Play!");
                playButton.CenterOn( G.SubGameRect.left + (0.5 * (G.SubGameRect.right - G.SubGameRect.left)),
                                     G.SubGameRect.top  + (0.7 * (G.SubGameRect.bottom - G.SubGameRect.top) ) );
                playButton.addEventListener(MouseEvent.CLICK, PlayButtonPressed);
            }
            playButton.visible = true;

            if (null == quitButton)
            {
                quitButton = InternalGameObjects.Add(PulsingTextButton) as PulsingTextButton;
                quitButton.SetFontColor(Colors.WHITE_ENOUGH);
                quitButton.SetHoverColor(Colors.WHITE);
                quitButton.SetFont(GameContent.TuffyFont);
                quitButton.SetFontSize(36);
                quitButton.SetText("back");
                quitButton.x = G.SubGameRect.left;
                quitButton.y = G.SubGameRect.bottom + 30;
                quitButton.addEventListener(MouseEvent.CLICK, QuitButtonPressed);
            }
            quitButton.visible = false;

            if (null == gameMask)
            {
                gameMask = new Sprite();
                gameMask.graphics.beginFill(0xFF00FF);
                gameMask.graphics.drawRect(
                    G.SubGameRect.left,
                    G.SubGameRect.top,
                    G.SubGameRect.width,
                    G.SubGameRect.height );
                gameMask.graphics.endFill();
                addChild(gameMask);
            }
        }

        private function PlayButtonPressed(e:MouseEvent = null):void
        {
            dispatchEvent(new Event(EVENT_PLAYBUTTON_PRESSED));
            playButton.visible = false;
            quitButton.visible = true;
            titleTextfield.visible = false;
            game.Init(G.SubGameRect.width, G.SubGameRect.height);
            game.ChangeGameState(_gameState);
        }

        private function QuitButtonPressed(e:MouseEvent = null):void
        {
            game.Shutdown();
            playButton.visible = true;
            quitButton.visible = false;
            titleTextfield.visible = true;
            dispatchEvent(new Event(EVENT_QUITBUTTON_PRESSED));
        }

        public override function Shutdown():void
        {
            super.Shutdown()
            game = null;
        }

        // helper function for kicking off transition animations
        private function Transition(startValues:Object, endValues:Object):void
        {
            Animate(startValues, endValues, TRANSITION_TIME);
        }

        public function TransitionOutLeft():void
        {
            var center:Object = {x:0, y:0};
            var left:Object = {x:-G.Game.StageWidth, y:0};
            Transition(center, left);
        }
        public function TransitionOutRight():void
        {
            var center:Object = {x:0, y:0};
            var right:Object = {x:0+G.Game.StageWidth, y:0};
            Transition(center, right);
        }
        public function TransitionOutUp():void
        {
            var center:Object = {x:0, y:0};
            var up:Object = {x:0, y:0-G.Game.StageHeight};
            Transition(center, up);
        }
        public function TransitionOutDown():void
        {
            var center:Object = {x:0, y:0};
            var down:Object = {x:0, y:0+G.Game.StageHeight};
            Transition(center, down);
        }
        public function TransitionInFromRight():void
        {
            var center:Object = {x:0, y:0};
            var right:Object = {x:0+G.Game.StageHeight, y:0};
            Transition(right, center);
        }
        public function TransitionInFromLeft():void
        {
            var center:Object = {x:0, y:0};
            var left:Object = {x:0-G.Game.StageHeight, y:0};
            Transition(left, center);
            visible = true;
        }
        public function TransitionInFromAbove():void
        {
            var center:Object = {x:0, y:0};
            var up:Object = {x:0, y:0-G.Game.StageWidth};
            Transition(up, center);
            visible = true;
        }
        public function TransitionInFromBelow():void
        {
            var center:Object = {x:0, y:0};
            var down:Object = {x:0, y:0+G.Game.StageWidth};
            Transition(down, center);
            visible = true;
        }



    }
}
