﻿package ld16.gameobjects
{
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import stego.core.GameObject;
    import flash.display.DisplayObject;
    import flash.geom.Rectangle;
    import stego.collision.CollisionInfo;
    import stego.constants.Keyboard;
    import stego.InputUtil;
    import stego.PlatformerCharacter;
    import stego.Sounds;
    import stego.Util;
    import stego.util.spriteanim.SpriteAnimUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Dude extends PlatformerCharacter
    {
        [Embed(source='../assets/man0.png')]
        private static const WALK_0:Class;
        [Embed(source='../assets/man1.png')]
        private static const WALK_1:Class;
        [Embed(source='../assets/man2.png')]
        private static const WALK_2:Class;
        [Embed(source='../assets/man3.png')]
        private static const WALK_3:Class;
        [Embed(source='../assets/man4.png')]
        private static const WALK_4:Class;
        [Embed(source='../assets/man5.png')]
        private static const WALK_5:Class;
        private static const WALK_FRAMES:Array = [WALK_5, WALK_1, WALK_2, WALK_3, WALK_4];

        private static const IDLE_FRAMES:Array = [WALK_0];
        private static const JUMP_FRAMES:Array = [WALK_2, WALK_3];

        [Embed(source='../assets/man_victory0.png')]
        private static const VICTORY_0:Class;
        [Embed(source='../assets/man_victory1.png')]
        private static const VICTORY_1:Class;
        [Embed(source='../assets/man_victory2.png')]
        private static const VICTORY_2:Class;
        [Embed(source='../assets/man_victory3.png')]
        private static const VICTORY_3:Class;
        private static const VICTORY_FRAMES:Array = [VICTORY_0, VICTORY_1, VICTORY_2, VICTORY_3, VICTORY_2, VICTORY_1];

        [Embed(source='../assets/man_lose.png')]
        private static const LOSE:Class;
        private static const LOSE_FRAMES:Array = [LOSE];

        private static var soundChannel:SoundChannel;
        [Embed(source='../assets/jump.mp3')]
        private static const Jump0SoundClass:Class;
        private static var Jump0Sound:Sound = new Jump0SoundClass();
        [Embed(source='../assets/jump1.mp3')]
        private static const Jump1SoundClass:Class;
        private static var Jump1Sound:Sound = new Jump1SoundClass();
        [Embed(source='../assets/jump2.mp3')]
        private static const Jump2SoundClass:Class;
        private static var Jump2Sound:Sound = new Jump2SoundClass();
        [Embed(source='../assets/jump3.mp3')]
        private static const Jump3SoundClass:Class;
        private static var Jump3Sound:Sound = new Jump3SoundClass();
        private static var JumpSounds:Array = [Jump0Sound, Jump1Sound, Jump2Sound, Jump3Sound];

        private var mAnimUtil:SpriteAnimUtil;

        private static const MOVE_RIGHT_KEY:Number = Keyboard.RIGHT;
        private static const MOVE_LEFT_KEY:Number = Keyboard.LEFT;
        private static const JUMP_KEYS:Array = [Keyboard.SPACE, Keyboard.SHIFT, Keyboard.UP, 90];

        // remember which key they used to jump, so we know which key's release event we care about
        private var mJumpKey:uint = 0;

        private var mIsJumping:Boolean = false;
        public function get IsJumping():Boolean { return mIsJumping; }
        public function set IsJumping(value:Boolean):void { mIsJumping = value; }

        private var _inputUtil:InputUtil;
        public function get inputUtil():InputUtil { return _inputUtil; }
        public function set inputUtil(value:InputUtil):void { _inputUtil = value; }

        private var _jumpSpeed:Number;
        public function get jumpSpeed():Number { return _jumpSpeed; }
        public function set jumpSpeed(value:Number):void { _jumpSpeed = value; }

        public override function Init():void
        {
            super.Init();

            mWalkAccel = 2000;
            mAirAccel = 2000;
            mStopAccel = 8000;
            mMaxWalkSpeed = 300;
            _jumpSpeed = 400;

            CreateGraphics();
            Restart();
        }

        protected function CreateGraphics():void
        {
            if (null == mAnimUtil)
            {
                mAnimUtil = new SpriteAnimUtil(this);
                mAnimUtil.scaleX = 2.0;
                mAnimUtil.scaleY = 2.0;
                mAnimUtil.AddAnimation("idle", IDLE_FRAMES);
                mAnimUtil.AddAnimation("walk", WALK_FRAMES, 16.0);
                mAnimUtil.AddAnimation("jump", JUMP_FRAMES, 3.0);
                mAnimUtil.AddAnimation("victory", VICTORY_FRAMES, 21.0);
                mAnimUtil.AddAnimation("lose", LOSE_FRAMES);
            }
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);
            mAnimUtil.Update(elapsedTime);

            if (PhysicsEnabled)
            {
                Walk(elapsedTime);
                Jumping(elapsedTime);
            }
        }

        /**
         * change dy based on user input
         */
        protected function Jumping(elapsedTime:Number):void
        {
            if (IsOnGround)
            {
                if (inputUtil.AnyJustPressed(JUMP_KEYS))
                {
                    dy = -jumpSpeed;
                    PlayJumpSound();
                }
            }
        }

        protected function PlayJumpSound():void
        {
            soundChannel = Sounds.PlaySound(Util.RandomOneOf(JumpSounds), soundChannel, 0.5);
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
            var desiredDirection:Number = 0;
            if (inputUtil.IsDown(MOVE_RIGHT_KEY))
            {
                desiredDirection = 1;
            }
            else if (inputUtil.IsDown(MOVE_LEFT_KEY))
            {
                desiredDirection = -1;
            }

            DoWalkAccel(desiredDirection, elapsedTime);

            // update ground animations now
            if (IsOnGround)
            {
                if ( 0 == desiredDirection )
                {
                    mAnimUtil.curAnim = "idle";
                }
                else
                {
                    mAnimUtil.curAnim = "walk";
                }
            }
            else
            {
                mAnimUtil.curAnim = "jump";
            }
        }

        private function DoWalkAccel(desiredDirection:Number, elapsedTime:Number):void
        {
            var currentDirection:Number = (dx>0)?1:(dx<0)?-1:0;
            // changing direction or stopping
            if ( desiredDirection != currentDirection )
            {
                var ddx:Number = StopAccel * elapsedTime;

                if ( ddx > Math.abs(dx) )
                {
                    dx = 0;
                }
                else
                {
                    if (dx > 0 )
                    {
                        dx -= ddx;
                    }
                    else
                    {
                        dx += ddx;
                    }
                }
            }

            dx += desiredDirection * WalkAccel * elapsedTime;

            // max speed
            if (dx > MaxWalkSpeed)
            {
                dx = MaxWalkSpeed;
            }
            if (dx < -MaxWalkSpeed)
            {
                dx = -MaxWalkSpeed;
            }

            // flip graphics
            if (desiredDirection > 0)
            {
                FaceRight();
            }
            if (desiredDirection < 0)
            {
                FaceLeft();
            }
        }

        public override function GetCollisionBounds(targetCoordinateSpace:DisplayObject = null):Rectangle
        {
            var bounds:Rectangle = super.GetCollisionBounds(targetCoordinateSpace);
            bounds.x += 0.25 * bounds.width;
            bounds.width *= 0.5;
            return bounds;
        }

        public function DoVictoryAnimation():void
        {
            mAnimUtil.curAnim = "victory";
        }

        public function DoLoseAnimation():void
        {
            mAnimUtil.curAnim = "lose";
        }

        public override function CanPassThrough(ci:CollisionInfo, object:GameObject):Boolean
        {
            return (object is Prize);
        }
    }
}
