﻿package ld16.gamestate
{
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import ld16.ArrowGraphic;
    import ld16.ArrowGraphicHover;
    import ld16.G;
    import ld16.GameContent;
    import ld16.OverworldScreen;
    import ld16.subgame.gamestate.Avoid;
    import ld16.subgame.gamestate.DeeKay;
    import ld16.subgame.gamestate.Dig;
    import ld16.subgame.gamestate.LessEasy;
    import ld16.subgame.gamestate.SuperEasy;
    import ld16.subgame.TestGameState;
    import ld16.widget.MiniMap;
    import stego.constants.Colors;
    import stego.core.GameInterface;
    import stego.core.GameState;
    import stego.Sounds;
    import stego.util.MathUtil;
    import stego.widget.PulsingGraphic;
    import stego.widget.PulsingGraphicButton;
    import stego.widget.TextField;


    /**
     * gamestate for browsing the "overworld" of game screens
     * @author Mike Meyer
     */
    public class Overworld extends GameState
    {
        public static const EVENT_WON:String = "won minigame";
        public static const EVENT_LOST:String = "lost minigame";

        private var overworldScreens:Array;
        private var numRows:Number;
        private var numCols:Number;
        private var overworldColumn:int = 0;
        private var overworldRow:int = 0;
        private var minimap:MiniMap;


        public function get currentScreen():OverworldScreen { return _currentScreen; }
        public function set currentScreen(value:OverworldScreen):void
        {
            if (null != _currentScreen)
            {
                _currentScreen.removeEventListener(Overworld.EVENT_WON, WonMiniGame);
                _currentScreen.removeEventListener(Overworld.EVENT_LOST, LostMiniGame);
            }
            _currentScreen = value;
            _currentScreen.addEventListener(Overworld.EVENT_WON, WonMiniGame);
            _currentScreen.addEventListener(Overworld.EVENT_LOST, LostMiniGame);
        }
        private var _currentScreen:OverworldScreen;

        private var rightButton:PulsingGraphicButton;
        private var leftButton:PulsingGraphicButton;
        private var upButton:PulsingGraphicButton;
        private var downButton:PulsingGraphicButton;
        private var hintTextField:TextField;
        private var levelSelectHintTextField:TextField;

        private static var soundChannel:SoundChannel;
        [Embed(source='../assets/winsound.mp3')]
        private static const WinSoundClass:Class;
        private static var WinSound:Sound = new WinSoundClass();
        [Embed(source='../assets/lose.mp3')]
        private static const LoseSoundClass:Class;
        private static var LoseSound:Sound = new LoseSoundClass();

        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            overworldScreens = CreateWorld();

            numRows = overworldScreens.length;
            numCols = 0;
            for each (var row:Array in overworldScreens)
            {
                numCols = Math.max(numCols, row.length);
            }

            CreateUI();
            GoToScreen(overworldColumn, overworldRow);
        }

        /**
         * create the array of arrays of OverworldScreens to represent the game worlds
         */
        private function CreateWorld():Array
        {
            overworldColumn = 0;
            overworldRow = 1;
            return [ [ null,
                       AddScreen(OverworldScreen, "boing", "whoa hey I didn't tell you to jump", new LessEasy()),
                       AddScreen(OverworldScreen, "shoot", "", new Dig()) ],
                     [ AddScreen(OverworldScreen, "super easy","use the arrow keys to move and jump", new SuperEasy()),
                       AddScreen(OverworldScreen, "avoid", "watch out for those things!", new Avoid()),
                       AddScreen(OverworldScreen, "dee kay", "I feel like I've done this before...", new DeeKay()) ] ];
        }

        private function AddScreen(screenClass:Class, title:String="<<<TODO>>>", hint:String="", gameState:GameState=null):OverworldScreen
        {
            var overworldScreen:OverworldScreen = Game.Layers.PlatformLayer.Add(screenClass) as OverworldScreen;
            overworldScreen.title = title;
            overworldScreen.hint = hint;
            overworldScreen.visible = false;
            overworldScreen.gameState = gameState;
            return overworldScreen;
        }

        private function CreateUI():void
        {
            rightButton = Game.Layers.UiLayer.Add(PulsingGraphicButton) as PulsingGraphicButton;
            rightButton.graphic = ArrowGraphic;
            rightButton.hoverGraphic = ArrowGraphicHover;
            rightButton.addEventListener(MouseEvent.CLICK, GoRight);
            rightButton.x = 578.75;
            rightButton.y = 268;

            leftButton = Game.Layers.UiLayer.Add(PulsingGraphicButton) as PulsingGraphicButton;
            leftButton.graphic = ArrowGraphic;
            leftButton.hoverGraphic = ArrowGraphicHover;
            leftButton.rotation = 180;
            leftButton.addEventListener(MouseEvent.CLICK, GoLeft);
            leftButton.x = 61.25;
            leftButton.y = 268;

            upButton = Game.Layers.UiLayer.Add(PulsingGraphicButton) as PulsingGraphicButton;
            upButton.graphic = ArrowGraphic;
            upButton.hoverGraphic = ArrowGraphicHover;
            upButton.rotation = 270;
            upButton.addEventListener(MouseEvent.CLICK, GoUp);
            upButton.x = 320;
            upButton.y = 41.25;

            downButton = Game.Layers.UiLayer.Add(PulsingGraphicButton) as PulsingGraphicButton;
            downButton.graphic = ArrowGraphic;
            downButton.hoverGraphic = ArrowGraphicHover;
            downButton.rotation = 90;
            downButton.addEventListener(MouseEvent.CLICK, GoDown);
            downButton.x = 320;
            downButton.y = 494.75;

            minimap = Game.Layers.UiLayer.Add(MiniMap) as MiniMap;
            minimap.x = Game.StageWidth - minimap.width;
            minimap.y = Game.StageHeight - minimap.height;
            minimap.data = overworldScreens;

            hintTextField = Game.Layers.UiLayer.Add(TextField) as TextField;
            hintTextField.SetFont(GameContent.TuffyFont);
            hintTextField.SetFontSize(24);
            hintTextField.SetFontColor(Colors.WHITE_ENOUGH);
            hintTextField.CenterOn(0.5 * (G.SubGameRect.left + G.SubGameRect.right), G.SubGameRect.bottom + 15);

            levelSelectHintTextField = Game.Layers.UiLayer.Add(TextField) as TextField;
            levelSelectHintTextField.SetFont(GameContent.TuffyFont);
            levelSelectHintTextField.SetFontSize(24);
            levelSelectHintTextField.SetFontColor(Colors.WHITE_ENOUGH);
            levelSelectHintTextField.CenterOn(0.5 * (G.SubGameRect.left + G.SubGameRect.right), 0.5 * G.SubGameRect.top);
            levelSelectHintTextField.SetText("Use the arrows to select a different game\nor click play to play this one.");
        }

        /**
         * move right a screen
         */
        private function GoRight(e:MouseEvent):void
        {
            currentScreen.TransitionOutLeft();
            GoToScreen(overworldColumn + 1, overworldRow);
            currentScreen.TransitionInFromRight();
            currentScreen.visible = true;
            levelSelectHintTextField.visible = false;
        }

        /**
         * move left a screen
         */
        private function GoLeft(e:MouseEvent):void
        {
            currentScreen.TransitionOutRight();
            GoToScreen(overworldColumn-1, overworldRow);
            currentScreen.TransitionInFromLeft();
            currentScreen.visible = true;
            levelSelectHintTextField.visible = false;
        }

        /**
         * move up a screen
         */
        private function GoUp(e:MouseEvent):void
        {
            currentScreen.TransitionOutDown();
            GoToScreen(overworldColumn, overworldRow-1);
            currentScreen.TransitionInFromAbove();
            currentScreen.visible = true;
            levelSelectHintTextField.visible = false;
        }

        /**
         * move down a screen
         */
        private function GoDown(e:MouseEvent):void
        {
            currentScreen.TransitionOutUp();
            GoToScreen(overworldColumn, overworldRow+1);
            currentScreen.TransitionInFromBelow();
            currentScreen.visible = true;
            levelSelectHintTextField.visible = false;
        }

        /**
         * jump to a screen of the game
         */
        private function GoToScreen(column:int, row:int):void
        {
            if (null != currentScreen)
            {
                currentScreen.removeEventListener(OverworldScreen.EVENT_PLAYBUTTON_PRESSED, PlayButtonPressed);
                currentScreen.removeEventListener(OverworldScreen.EVENT_QUITBUTTON_PRESSED, QuitButtonPressed);
            }

            overworldRow = MathUtil.Wrap(row, 0, numRows);
            overworldColumn = MathUtil.Wrap(column, 0, numCols);
            currentScreen = GetScreenAt(column, row);
            currentScreen.addEventListener(OverworldScreen.EVENT_PLAYBUTTON_PRESSED, PlayButtonPressed);
            currentScreen.addEventListener(OverworldScreen.EVENT_QUITBUTTON_PRESSED, QuitButtonPressed);
            currentScreen.visible = true;
            UpdateArrows();

            minimap.currentRow = overworldRow;
            minimap.currentColumn = overworldColumn;
        }

        private function PlayButtonPressed(e:Event):void
        {
            ShowHint(currentScreen.hint);
            HideArrows();
            minimap.visible = false;
            levelSelectHintTextField.visible = false;
        }

        private function HideArrows():void
        {
            rightButton.visible = false;
            leftButton.visible  = false;
            upButton.visible    = false;
            downButton.visible  = false;
        }

        private function ShowHint(text:String):void
        {
            hintTextField.visible = true;
            hintTextField.SetText(text);
        }
        private function QuitButtonPressed(e:Event):void
        {
            hintTextField.visible = false;
            UpdateArrows();
            minimap.visible = true;
        }

        /**
         * make sure that only arrows that take you somewhere are visible
         */
        private function UpdateArrows():void
        {
            rightButton.visible = (null != GetScreenAt(overworldColumn + 1, overworldRow));
            leftButton.visible  = (null != GetScreenAt(overworldColumn - 1, overworldRow));
            upButton.visible    = (null != GetScreenAt(overworldColumn, overworldRow - 1));
            downButton.visible  = (null != GetScreenAt(overworldColumn, overworldRow + 1));
        }

        private function GetScreenAt(column:int, row:int):OverworldScreen
        {
            var rowIndex:Number = MathUtil.Wrap(row, 0, numRows);
            var rowOfScreens:Array = overworldScreens[rowIndex];
            var colIndex:Number = MathUtil.Wrap(column, 0, numCols);
            return rowOfScreens[colIndex];
        }

        public override function End():void
        {
            Game.Layers.RemoveAll();
            super.End();
        }

        private function WonMiniGame(e:Event):void
        {
            soundChannel = Sounds.PlaySound(WinSound, soundChannel, 0.5);
        }

        private function LostMiniGame(e:Event):void
        {
            soundChannel = Sounds.PlaySound(LoseSound, soundChannel, 0.5);
        }
    }
}
