﻿package ld16.subgame.gamestate
{
    import ld16.G;
    import ld16.gameobjects.Prize;
    import ld16.gameobjects.Spiky;
    import stego.script.step.Animate;
    import stego.script.step.Loop;
    import stego.script.step.Wait;
    import stego.script.step.WaitTime;

    /**
     * ...
     * @author Mike Meyer
     */
    public class DeeKay extends SuperEasy
    {
        private var highSpiky:Spiky;
        private var highSpikyWent:Boolean;
        private var sideSpiky:Spiky;

        protected override function CreatePlayer():void
        {
            super.CreatePlayer();
            player.x = player.width;
            player.y = G.SubGameRect.height - player.height;
            player.scaleX = 0.5;
            player.scaleY = 0.5;
        }

        protected override function CreatePlatforms():void
        {
            var y:Number = G.SubGameRect.height;

            var xValues:Array = [ 0.15 * G.SubGameRect.width,
                                  0.90 * G.SubGameRect.width,
                                  0.90 * G.SubGameRect.width,
                                  0.0 * G.SubGameRect.width,
                                  0.0 * G.SubGameRect.width,
                                  0.0 * G.SubGameRect.width ];
            var widths:Array =  [ 0.85 * G.SubGameRect.width,
                                  0.1 * G.SubGameRect.width,
                                  0.1 * G.SubGameRect.width ];
            var ysteps:Array =  [ 16,
                                  16,
                                  32 ];

            var i:Number = 0;
            while (y > 32)
            {
                PlacePlatform(
                    xValues[i],
                    y,
                    widths[i%widths.length],
                    16);
                y -= ysteps[i%ysteps.length];
                i = (i + 1) % xValues.length;
            }
        }

        protected override function CreatePrize():void
        {
            var prize:Prize = Game.Layers.ItemLayer.Add(Prize) as Prize;
            prize.x = 0.3 * G.SubGameRect.width;
            prize.y = 32;
            prize.dx = -500;
            prize.dy = -300;
        }

        protected override function CreateSpikies():void
        {
            super.CreateSpikies();

            highSpikyWent = false;
            highSpiky = Game.Layers.EnemyLayer.Add(Spiky) as Spiky;
            highSpiky.x = G.SubGameRect.width + highSpiky.width;
            highSpiky.y = 24;
            highSpiky.PhysicsEnabled = false;
            highSpiky.AffectedByGravity = false;

            sideSpiky = Game.Layers.EnemyLayer.Add(Spiky) as Spiky;
            sideSpiky.PhysicsEnabled = false;
            sideSpiky.AffectedByGravity = false;
            sideSpiky.scheduler.RunScript(new Loop( -1, [new Animate(sideSpiky, { x:8, y:24 }, { x:8, y:G.SubGameRect.height }, 3.0),
                                                         new Animate(sideSpiky, { x: -highSpiky.width, y:24 }, { x:0.5 * G.SubGameRect.width, y:24 }, 1.0),
                                                         new Animate(sideSpiky, { y:24 }, { y:G.SubGameRect.height }, 1.0)] ))
        }

        public override function Update():void
        {
            super.Update();

            if (!highSpikyWent)
            {
                if (player.y < 60)
                {
                    highSpikyWent = true;
                    highSpiky.scheduler.RunScript( new Loop(1, [ new WaitTime(0.4),
                                                                 new Animate(highSpiky, {x:-highSpiky.width}, {x:G.SubGameRect.width + highSpiky.width}, 1.75) ] ) );
                }
            }
        }
    }

}
