﻿package ld16.subgame.gamestate
{
    import ld16.G;
    import ld16.gameobjects.Destructable;
    import ld16.gameobjects.Dude;
    import stego.collision.CollisionInfo;
    import stego.constants.Colors;
    import stego.constants.Keyboard;
    import stego.core.BaseGame;
    import stego.core.GameObject;
    import stego.InputUtil;
    import stego.script.step.Loop;
    import stego.script.step.RunFunc;
    import stego.script.step.WaitTime;
    import stego.Util;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Dig extends SuperEasy
    {
        protected override function CreatePlayer():void
        {
            player = Game.Layers.PlayerLayer.Add(Dude) as Dude;
            player.inputUtil = Game.inputUtil;
            player.x = 0.1 * G.SubGameRect.width;
            player.y = 0.1 * G.SubGameRect.height;
            player.jumpSpeed = 0;
        }

        protected override function CreatePlatforms():void
        {
            var rowHeight:Number = 96;
            var platformThickness:Number = 32;
            var y:Number = rowHeight;
            var startingX:Number = 0;
            var moveDelay:Number = 1.0;
            var moveDelayStep:Number = 0.3;
            while (y < G.SubGameRect.height)
            {
                PlacePlatform(
                    startingX,
                    y,
                    0.8 * G.SubGameRect.width,
                    platformThickness);

                startingX = 0.2 * G.SubGameRect.width - startingX;

                for (var x:Number = 0.3 * G.SubGameRect.width; x < 0.7 * G.SubGameRect.width; x += 32 )
                {
                    var destructable:Destructable = Game.Layers.EnemyLayer.Add(Destructable) as Destructable;
                    destructable.x = x;
                    destructable.y = y - 16;

                    ScheduleMove(destructable, moveDelay);
                    moveDelay += moveDelayStep;
                }

                y += rowHeight;
            }
        }

        private function ScheduleMove(gameObject:GameObject, delay:Number):void
        {
            gameObject.scheduler.RunScript(new Loop( 1, [
                new Loop( -1, [
                    new WaitTime(delay),
                    new RunFunc(function():void {
                        gameObject.AnimateTo( { x:Math.random() * G.SubGameRect.width,
                                                y:Math.random() * G.SubGameRect.height}, 1.0)}),
                        ])]));
        }

        public override function Update():void
        {
            super.Update();

            if (Game.inputUtil.WasJustPressed(Keyboard.UP))
            {
                FireBullet();
            }
            CheckBulletCollisions();
            KillDeadBullets();
        }

        private function KillDeadBullets():void
        {
            for each (var bullet:GameObject in Game.Layers.BulletLayer.Objects)
            {
                if (! bullet.GetCollisionBounds(Game as BaseGame).intersects(Game.StageRect))
                {
                    bullet.isAlive = false;
                }
            }
        }

        private function CheckBulletCollisions():void
        {
            Game.Layers.ForEachCartesianProductStyle(Game.Layers.BulletLayer, Game.Layers.EnemyLayer, BulletDestructableCollisions);
        }

        private function BulletDestructableCollisions(bullet:GameObject, enemy:GameObject):void
        {
            var ci:CollisionInfo = Util.CheckAabbCollision(bullet, enemy, Game.Time.TimeDelta);

            if (null != ci)
            {
                if (enemy is Destructable)
                {
                    enemy.isAlive = false;
                    bullet.isAlive = false;
                }
            }
        }

        private function FireBullet():void
        {
            var bulletSpeed:Number = 800;
            var bulletWidth:Number = 32;
            var bulletHeight:Number = 4;

            var direction:Number = player.scaleX;
            var bullet:GameObject = Game.Layers.BulletLayer.Add(GameObject);
            bullet.x = player.x;
            bullet.y = player.y - 8;
            bullet.dx = bulletSpeed * direction;
            bullet.dy = 100;
            bullet.graphics.beginFill(Colors.RED);
            bullet.graphics.drawRect(
                -0.5 * bulletWidth,
                -0.5 * bulletHeight,
                bulletWidth,
                bulletHeight );
            bullet.graphics.endFill();
            bullet.AffectedByGravity = false;
            bullet.PhysicsEnabled = false;
        }
    }

}
