﻿package ld16.subgame.gamestate
{
    import adobe.utils.CustomActions;
    import flash.events.Event;
    import ld16.G;
    import ld16.gameobjects.Dude;
    import ld16.gameobjects.Platform;
    import ld16.gameobjects.Prize;
    import ld16.gameobjects.Spiky;
    import ld16.gamestate.Overworld;
    import stego.core.GameInterface;
    import stego.core.GameObject;
    import stego.core.GameState;
    import stego.Util;

    /**
     * ...
     * @author Mike Meyer
     */
    public class SuperEasy extends GameState
    {
        protected var player:Dude;

        private var _isGameOver:Boolean;
        public function get isGameOver():Boolean { return _isGameOver; }

        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            _isGameOver = false;

            // walls to keep him on screen
            var thickness:Number = 64;

            // floor
            PlacePlatform(-thickness, G.SubGameRect.height, G.SubGameRect.width + thickness, thickness);
            // right
            PlacePlatform(G.SubGameRect.width, 0, thickness, G.SubGameRect.height + thickness);
            // top
            PlacePlatform(0, -thickness, G.SubGameRect.width + thickness, thickness);
            // left
            PlacePlatform( -thickness, -thickness, thickness, G.SubGameRect.height + thickness);

            CreatePlatforms();
            CreatePlayer();
            CreatePrize();
            CreateSpikies();
        }

        protected function CreatePlatforms():void
        {
            // thing to jump over
            var obstacleHeight:Number = 32;
            PlacePlatform(
                0.47 * G.SubGameRect.width,
                G.SubGameRect.height - obstacleHeight,
                0.06 * G.SubGameRect.width,
                obstacleHeight);
        }

        protected function CreatePlayer():void
        {
            player = Game.Layers.PlayerLayer.Add(Dude) as Dude;
            player.inputUtil = Game.inputUtil;
            player.x = 0.2 * G.SubGameRect.width;
            player.y = 0.8 * G.SubGameRect.height;
        }

        protected function CreatePrize():void
        {
            var prize:Prize = Game.Layers.ItemLayer.Add(Prize) as Prize;
            prize.x = 0.8 * G.SubGameRect.width;
            prize.y = G.SubGameRect.height - prize.height;
            prize.dy = -800;
        }

        public function PlacePlatform(x:Number, y:Number, width:Number, height:Number):void
        {
            var platform:Platform = Game.Layers.PlatformLayer.Add(Platform) as Platform;
            platform.Place(x, y, width, height);
        }

        public override function Update():void
        {
            super.Update();
            if (!isGameOver)
            {
                var allPrizesCollected:Boolean = true;
                for each (var prize:Prize in Game.Layers.ItemLayer.Objects)
                {
                    if (null != Util.CheckAabbCollision(player, prize, Game.Time.TimeDelta))
                    {
                        prize.visible = false;
                    }
                    if (prize.visible)
                    {
                        allPrizesCollected = false;
                    }
                }
                if (allPrizesCollected)
                {
                    GameOver();
                    dispatchEvent(new Event(Overworld.EVENT_WON));
                    player.DoVictoryAnimation();
                }

                for each (var enemy:GameObject in Game.Layers.EnemyLayer.Objects)
                {
                    if (null != Util.CheckAabbCollision(player, enemy, Game.Time.TimeDelta))
                    {
                        player.DoLoseAnimation();
                        dispatchEvent(new Event(Overworld.EVENT_LOST));
                        GameOver();
                        return;
                    }
                }
            }
        }

        public override function End():void
        {
            Game.Layers.RemoveAll();
            super.End();
        }

        protected function GameOver():void
        {
            player.PhysicsEnabled = false;
            player.AffectedByGravity = false;
            player.dx = 0;
            player.dy = 0;
            _isGameOver = true;
        }

        protected function CreateSpikies():void
        {
        }
    }

}
