﻿package ld16.widget
{
    import flash.display.Sprite;
    import flash.geom.Rectangle;
    import ld16.OverworldScreen;
    import stego.constants.Colors;
    import stego.core.GameObject;

    /**
     * ...
     * @author Mike Meyer
     */
    public class MiniMap extends GameObject
    {
        private static const WIDTH:Number = 200;
        private static const HEIGHT:Number = 100;
        private var background:Sprite;
        private var roomsHolder:Sprite;
        private var roomRows:Array;
        private var youAreHereIndicator:Sprite;

        private var _data:Array;
        public function set data(value:Array):void { _data = value; BuildMap(); }
        public function get data():Array { return _data; }

        public function get currentRow():uint { return _currentRow; }
        public function set currentRow(value:uint):void { _currentRow = value; UpdateCurrentPos();  }
        private var _currentRow:uint;

        public function get currentColumn():uint { return _currentColumn; }
        public function set currentColumn(value:uint):void { _currentColumn = value; UpdateCurrentPos(); }
        private var _currentColumn:uint;

        /**
         * initial setup and preparation for reuse
         */
        public override function Init():void
        {
            super.Init();

            CreateGraphics();
        }

        /**
         * create the graphics
         */
        private function CreateGraphics():void
        {
            if (null == background)
            {
                background = new Sprite();
                background.graphics.lineStyle(2, Colors.WHITE_ENOUGH);
                background.graphics.beginFill(Colors.BLACK_ENOUGH);
                background.graphics.drawRect(0, 0, WIDTH, HEIGHT);
                background.graphics.endFill();
                addChild(background);
            }

            if (null == roomsHolder)
            {
                roomsHolder = new Sprite();
                addChild(roomsHolder);
            }

            if (null == youAreHereIndicator)
            {
                youAreHereIndicator = new Sprite();
                youAreHereIndicator.graphics.beginFill(Colors.YELLOW);
                youAreHereIndicator.graphics.drawCircle(0,0,10);
                youAreHereIndicator.graphics.endFill();
                addChild(youAreHereIndicator);
            }
        }

        /**
         *
         */
        public function BuildMap():void
        {
            RemoveAllRooms();

            var numRows:uint = data.length;
            var numCols:uint = 0;
            var row:Array;
            for each (row in data)
            {
                numCols = Math.max(numCols, row.length);
            }

            roomRows = [];

            var flashsBordersSuckAdjustment:Number = 1;

            var roomWidth:Number = (WIDTH - Room.MARGIN - flashsBordersSuckAdjustment)/ numCols;
            var roomHeight:Number = (HEIGHT - Room.MARGIN - flashsBordersSuckAdjustment) / numRows;
            for (var rowIndex:uint = 0; rowIndex < data.length; ++rowIndex )
            {
                row = data[rowIndex];
                var dataRow:Array = _data[rowIndex];
                var roomRow:Array = [];
                for (var colIndex:uint = 0; colIndex < row.length; ++colIndex )
                {
                    var roomData:OverworldScreen = dataRow[colIndex];
                    if (null != roomData)
                    {
                        var room:Room = new Room(roomWidth, roomHeight);
                        room.x = colIndex * roomWidth + (0.5 * Room.MARGIN);
                        room.y = rowIndex * roomHeight + (0.5 * Room.MARGIN);
                        roomsHolder.addChild(room);
                        roomRow.push(room);
                    }
                    else
                    {
                        roomRow.push(null);
                    }
                }
                roomRows.push(roomRow);
            }
        }

        /**
         * remove all room objects from the minimap
         */
        private function RemoveAllRooms():void
        {
            while (0 < roomsHolder.numChildren)
            {
                roomsHolder.removeChildAt(roomsHolder.numChildren-1);
            }
        }

        /**
         * move the "you are here" indicator
         */
        private function UpdateCurrentPos():void
        {
            var roomRect:Rectangle = GetRoom(_currentColumn, _currentRow).getBounds(youAreHereIndicator.parent);
            youAreHereIndicator.width = Math.min(roomRect.width, roomRect.height) * 0.5;
            youAreHereIndicator.height = youAreHereIndicator.width;
            youAreHereIndicator.x = roomRect.x + 0.5 * roomRect.width;
            youAreHereIndicator.y = roomRect.y + 0.5 * roomRect.height;
        }

        private function GetRoom(column:uint, row:uint):Room
        {
            return roomRows[row][column];
        }
    }

}
