﻿package stego
{
    import flash.display.DisplayObject;
    import flash.display.MovieClip;
    import flash.display.Sprite;
    import flash.display.StageAlign;
    import flash.display.StageScaleMode;
    import flash.events.Event;
    import flash.utils.getDefinitionByName;
    import stego.core.GameInterface;

    /**
     * Base preloader. Extend and set as your startup class.
     *
     * game-specific implementations probably need nothing more than
     * public function Preloader()
     * {
     *     super(<width>, <height>, <name of "Main" class>);
     * }
     */
    public class Preloader extends Sprite
    {
        private static const LOADING_BAR_WIDTH:Number = 300;
        private static const LOADING_BAR_HEIGHT:Number = 30;
        private static const LOADING_BAR_BORDERWIDTH:Number = 2;
        private static const LOADING_BAR_MARGIN:Number = 2;
        private static const LOADING_BAR_COLOR:Number = 0xFFFFFF; // TODO: should come from BaseGame or better yet some constants file

        private var mLoadingBarBorder:Sprite;
        private var mLoadingBarProgress:Sprite;

        // working around an apparent Flash bug
        private var mPreviousProgress:Number;

        protected var stageWidth:Number;
        protected var stageHeight:Number;
        protected var className:String;

        /**
         * constructor
         * @param	stageWidth  - width of the game
         * @param	stageHeight - height of the game
         * @param	className   - name of the "main" game class to instantiate once everything has been preloaded
         */
        public function Preloader(stageWidth:Number = 480, stageHeight:Number = 320, className:String = "stego.core.BaseGame")
        {
            this.stageWidth = stageWidth;
            this.stageHeight = stageHeight;
            this.className = className;

            addEventListener(Event.ENTER_FRAME, checkProgress);

            mPreviousProgress = 0;

            // show loader
            createGraphics();
        }

        /**
         * called every frame to update the progress bar and check for when loading is complete
         */
        private function checkProgress(e:Event):void
        {
            var p:Number = (this.loaderInfo.bytesLoaded / this.loaderInfo.bytesTotal);

            // if p seems to have gotten invalid, just skip the preloader stuff
            if ( (mPreviousProgress > p) || (p >= 1.0) )
            {
                removeEventListener(Event.ENTER_FRAME, checkProgress);
                startup();
            }
            else
            {
                mPreviousProgress = p;

                mLoadingBarProgress.scaleX = p;
            }
        }

        /**
         * once preloading is complete, start the game up
         */
        private function startup():void
        {
            // hide loader
            removeGraphics();

            // mask out anything that would go outside of the game area
            var mask:Sprite = new Sprite();
            mask.graphics.beginFill(0xFF00FF);
            mask.graphics.drawRect(
                0,
                0,
                stageWidth,
                stageHeight );
            mask.graphics.endFill();
            addChild(mask);

            var mainClass:Class = getDefinitionByName(className) as Class;
            var mainAsDo:DisplayObject = new mainClass();
            var mainAsGame:GameInterface = mainAsDo as GameInterface;
            addChild(mainAsDo);
            mainAsGame.Init(stageWidth, stageHeight);
            mainAsDo.mask = mask;
        }

        private function createGraphics():void
        {
            var barX:Number = 0.5 * ( stageWidth - LOADING_BAR_WIDTH);
            var barY:Number = 0.5 * ( stageHeight - LOADING_BAR_HEIGHT);

            mLoadingBarBorder = new Sprite();
            mLoadingBarBorder.graphics.lineStyle(LOADING_BAR_BORDERWIDTH, LOADING_BAR_COLOR, 1);
            mLoadingBarBorder.graphics.beginFill(LOADING_BAR_COLOR, 0.0);
            mLoadingBarBorder.graphics.drawRect(
                0,
                0,
                LOADING_BAR_WIDTH,
                LOADING_BAR_HEIGHT);
            mLoadingBarBorder.graphics.endFill();
            mLoadingBarBorder.x = barX;
            mLoadingBarBorder.y = barY;
            this.addChild(mLoadingBarBorder);

            mLoadingBarProgress = new Sprite();
            mLoadingBarBorder.graphics.lineStyle(LOADING_BAR_BORDERWIDTH, LOADING_BAR_COLOR, 0);
            mLoadingBarProgress.graphics.beginFill(LOADING_BAR_COLOR);
            mLoadingBarProgress.graphics.drawRect(
                0,
                0,
                LOADING_BAR_WIDTH - (2.0 * LOADING_BAR_MARGIN),
                LOADING_BAR_HEIGHT - (2.0 * LOADING_BAR_MARGIN) );
            mLoadingBarProgress.graphics.endFill();
            mLoadingBarProgress.x = barX + LOADING_BAR_MARGIN;
            mLoadingBarProgress.y = barY + LOADING_BAR_MARGIN;
            this.addChild(mLoadingBarProgress);
        }

        private function removeGraphics():void
        {
            this.removeChild(mLoadingBarBorder);
            mLoadingBarBorder = null;
            this.removeChild(mLoadingBarProgress);
            mLoadingBarProgress = null;
        }
    }
}
