﻿package stego.core
{
    import flash.events.Event;
    import flash.events.EventDispatcher;
    import stego.core.GameInterface;
    import stego.script.Step;
    import stego.script.step.Loop;
    import stego.script.step.Parallel;
    import stego.script.step.RunFunc;
    import stego.script.step.Wait;

    /**
     * ...
     * @author Mike Meyer
     */
    public class GameState extends EventDispatcher
    {
        public static var EVENT_BEAT:String    = "beat";
        public static var EVENT_OFFBEAT:String = "offbeat";
        public static var EVENT_QUARTERBEAT:String = "quarterbeat";

        private var mTimeSinceLastQuarterBeat:Number;
        private var mCurQuarterBeat:Number;

        private var _game:GameInterface;
        public function get Game():GameInterface { return _game; }

        protected var mScript:Step;

        public function GameState()
        {
            mTimeSinceLastQuarterBeat = 0;
            mCurQuarterBeat = 0;
        }

        /**
         * "heartbeat" function. called every frame
         */
        public function Update():void
        {
            mTimeSinceLastQuarterBeat += Game.Time.TimeDelta;

            if (mTimeSinceLastQuarterBeat*4 >= Game.Time.SecondsPerBeat)
            {
                QuarterBeat();
            }
        }

        /**
         * last-minute updating before things get rendered each frame
         */
        public function PreDraw():void
        {
        }

        /**
         * called every 1/4 beat (from Update)
         */
        private function QuarterBeat():void
        {
            mTimeSinceLastQuarterBeat = 0;
            ++mCurQuarterBeat;

            // send event
            if (1 == (mCurQuarterBeat % 4))
            {
                // execute next step
                DoBeatAction(GetCurBeat())

                dispatchEvent(new Event(EVENT_BEAT));
            }
            else if (3 == (mCurQuarterBeat % 4))
            {
                dispatchEvent(new Event(EVENT_OFFBEAT));
            }

            dispatchEvent(new Event(EVENT_QUARTERBEAT));
        }

        /**
         * override me!
         * Do whatever is supposed to happen at this beat
         */
        protected function DoBeatAction(curBeat:Number):void
        {
            if (null != mScript)
            {
                mScript.Execute(Game.Time.SecondsPerBeat);
            }
        }

        /**
         * Get the beat we are currently on
         */
        public function GetCurBeat():Number
        {
            return Math.ceil(mCurQuarterBeat / 4);
        }

        /**
         * Get the part of the beat we are currently on
         */
        public function GetCurQuarterBeat():Number
        {
            return mCurQuarterBeat;
        }

        /**
         * Set the current beat
         */
        public function SetCurBeat(beat:Number):void
        {
            mCurQuarterBeat = 1 + (beat * 4);
        }

        /**
         * Go back to the beginning
         */
        public function Restart():void
        {
            mCurQuarterBeat = 0;
            End();
            Begin(Game);
            if (null != mScript)
            {
                mScript.Restart();
            }
        }

        /**
         * get the gamestate ready to be played
         */
        public function Begin(game:GameInterface):void
        {
            _game = game;

            if (null != mScript)
            {
                mScript.Restart();
            }
        }

        /**
         * clean up after the script
         */
        public function End():void
        {
            if (null != mScript)
            {
                mScript.Restart();
                mScript = null;
            }
        }
    }
}
