package stego.script.step.animate
{
    import flash.display.DisplayObject;
    import stego.util.MathUtil;
    public class AnimationValues
    {
        private var _animateAlpha:Boolean = false;
        private var _alpha:Number;
        public function get AnimateAlpha():Boolean { return _animateAlpha;}
        public function get alpha():Number { return _alpha; }
        public function set alpha(value:Number):void { _animateAlpha = true; _alpha = value; }

        private var _animateX:Boolean = false;
        private var _x:Number;
        public function get AnimateX():Boolean { return _animateX;}
        public function get x():Number { return _x; }
        public function set x(value:Number):void { _animateX = true; _x = value; }

        private var _animateY:Boolean = false;
        private var _y:Number;
        public function get AnimateY():Boolean { return _animateY;}
        public function get y():Number { return _y; }
        public function set y(value:Number):void { _animateY = true; _y = value; }

        private var _animateRotation:Boolean = false;
        private var _rotation:Number;
        public function get AnimateRotation():Boolean { return _animateRotation;}
        public function get rotation():Number { return _rotation; }
        public function set rotation(value:Number):void { _animateRotation = true; _rotation = value; }

        private var _animateScaleX:Boolean = false;
        private var _scaleX:Number;
        public function get AnimateScaleX():Boolean { return _animateScaleX;}
        public function get scaleX():Number { return _scaleX; }
        public function set scaleX(value:Number):void { _animateScaleX = true; _scaleX = value; }

        private var _animateScaleY:Boolean = false;
        private var _scaleY:Number;
        public function get AnimateScaleY():Boolean { return _animateScaleY;}
        public function get scaleY():Number { return _scaleY; }
        public function set scaleY(value:Number):void { _animateScaleY = true; _scaleY = value; }

        /**
         * make a new AnimationValues object from an object
         */
        public static function BuildFromObject(values:Object):AnimationValues
        {
            var av:AnimationValues = new AnimationValues();

            for (var property:String in values)
            {
                switch (property)
                {
                    case "alpha":
                        av.alpha = values[property];
                        break;
                    case "x":
                        av.x = values[property];
                        break;
                    case "y":
                        av.y = values[property];
                        break;
                    case "rotation":
                        av.rotation = values[property];
                        break;
                    case "scaleX":
                        av.scaleX = values[property];
                        break;
                    case "scaleY":
                        av.scaleY = values[property];
                        break;
                    default:
                        throw new Error("unrecognized property:  \"" + property + "\"");
                        break;
                }
            }

            return av;
        }

        /**
         * make a new AnimationValues object from an object
         */
        public static function BuildFromDisplayObject(displayObject:DisplayObject):AnimationValues
        {
            var av:AnimationValues = new AnimationValues();
            av.alpha = displayObject.alpha;
            av.x = displayObject.x;
            av.y = displayObject.y;
            av.rotation = displayObject.rotation;
            av.scaleX = displayObject.scaleX;
            av.scaleY = displayObject.scaleY;
            return av;
        }

        /**
         * Lerp a DisplayObject's properties
         * @param	start
         * @param	end
         * @param	time
         * @param	duration
         */
        public static function AnimateBetween(
            target:DisplayObject,
            start:AnimationValues,
            end:AnimationValues,
            time:Number,
            duration:Number):void
        {
            var t:Number = time / duration;
            t = MathUtil.Clamp(t, 0, 1);

            if (start.AnimateAlpha && end.AnimateAlpha)
            {
                target.alpha = MathUtil.Lerp(start.alpha, end.alpha, t);
            }
            if (start.AnimateX && end.AnimateX)
            {
                target.x = MathUtil.Lerp(start.x, end.x, t);
            }
            if (start.AnimateY && end.AnimateY)
            {
                target.y = MathUtil.Lerp(start.y, end.y, t);
            }
            if (start.AnimateRotation && end.AnimateRotation)
            {
                target.rotation = MathUtil.Lerp(start.rotation, end.rotation, t);
            }
            if (start.AnimateScaleX && end.AnimateScaleX)
            {
                target.scaleX = MathUtil.Lerp(start.scaleX, end.scaleX, t);
            }
            if (start.AnimateScaleY && end.AnimateScaleY)
            {
                target.scaleY = MathUtil.Lerp(start.scaleY, end.scaleY, t);
            }
        }
    }
}
