﻿package stego.util
{
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import stego.collision.CollisionInfo;
    import stego.core.GameObject;
    import stego.PlatformerCharacter;
    import stego.Settings;
    import stego.Util;
    import stego.util.SceneLayers;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PhysicsUtil
    {
        private var timeDelta:Number;

        /**
         * Run a physics step of the simulation
         */
        public function Update(layers:SceneLayers, timeDelta:Number):void
        {
            this.timeDelta = timeDelta;
            ApplyGravity(layers);
            Collisions(layers);
        }

        /**
         * detect and react to collisions
         */
        protected function Collisions(layers:SceneLayers):void
        {
            layers.PlayerLayer.ForEach(CallPreCollisionCheck);
            layers.NpcLayer.ForEach(CallPreCollisionCheck);
            layers.EnemyLayer.ForEach(CallPreCollisionCheck);
            layers.ItemLayer.ForEach(CallPreCollisionCheck);

            layers.ForEachCartesianProductStyle(layers.PlayerLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.PlayerLayer, layers.ItemLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.NpcLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.EnemyLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.ItemLayer, layers.PlatformLayer, CharacterPlatformCollision);

            layers.PlayerLayer.ForEach(CallPostCollisionCheck);
            layers.NpcLayer.ForEach(CallPostCollisionCheck);
            layers.EnemyLayer.ForEach(CallPostCollisionCheck);
            layers.ItemLayer.ForEach(CallPostCollisionCheck);
        }

        /**
         */
        protected function CallPreCollisionCheck(go:GameObject):void
        {
            if (go is PlatformerCharacter)
            {
                var character:PlatformerCharacter = go as PlatformerCharacter;
                character.PreCollisionCheck(timeDelta);
            }
        }
        /**
         */
        protected function CallPostCollisionCheck(go:GameObject):void
        {
            if (go is PlatformerCharacter)
            {
                var character:PlatformerCharacter = go as PlatformerCharacter;
                character.PostCollisionCheck(timeDelta);
            }
        }

        /**
         * detect and react to the player colliding with platforms
         */
        protected function CharacterPlatformCollision(go:GameObject, platform:GameObject):void
        {
            if (go.PhysicsEnabled && platform.PhysicsEnabled)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(go, platform, timeDelta);

                if (null != ci)
                {
                    var converted:CollisionInfo = ci.GlobalToLocal(go.parent);

                    // let the character know he hit something
                    go.CollisionOccurred(converted, platform);
                }
            }
        }

        /**
         * everything that can fall needs to have gravity applied
         */
        protected function ApplyGravity(layers:SceneLayers):void
        {
            layers.NpcLayer.ForEach(ApplyGravityTo);
            layers.PlayerLayer.ForEach(ApplyGravityTo);
            layers.EnemyLayer.ForEach(ApplyGravityTo);
            layers.BloodLayer.ForEach(ApplyGravityTo);
            layers.ItemLayer.ForEach(ApplyGravityTo);
        }

        /**
         * apply gravity to the object
         */
        protected function ApplyGravityTo(go:GameObject):void
        {
            if (go.AffectedByGravity)
            {
                go.dy += Settings.Gravity * timeDelta;

                if (go.dy > Settings.MaxFallSpeed)
                {
                    go.dy = Settings.MaxFallSpeed;
                }
            }
        }
    }

}
