﻿package stego.util.spriteanim
{
    import flash.display.DisplayObject;

    /**
     * struct used by SpriteAnimUtil.  Not very useful outside of that.
     * @author Mike Meyer
     */
    public class SpriteAnim
    {
        /**
         * get/set the number of frames to play each second
         */
        public function get curFrame():uint { return _curFrame; }
        public function set curFrame(value:uint):void { _curFrame = value; updateFrameVisibility(); }
        private var _curFrame:uint;

        /**
         * use to toggle visibility of this animation
         */
        public function get visible():Boolean { return _visible; }
        public function set visible(value:Boolean):void { _visible = value; updateFrameVisibility();  }
        private var _visible:Boolean = true;

        /**
         * get/set the array of display objects it uses as frames
         */
        public var frames:/*DisplayObject*/Array;

        /**
         * get/set the number of frames to play each second
         */
        public var framerate:Number = 4.0;

        /**
         * count up the time until the next frame switch
         */
        private var timer:Number;

        /**
         * constructor
         */
        public function SpriteAnim()
        {
            _curFrame = 0;
            frames = [];
            timer = 0;
        }

        /**
         * call every frame to make the animations go
         * @param	elapsedTime - time since the last call
         */
        public function update(elapsedTime:Number):void
        {
            timer += elapsedTime;

            var timeBetweenFrames:Number = 1.0 / framerate;
            var framesToAdvance:Number = Math.floor(timer / timeBetweenFrames);

            if ( (0 < framesToAdvance) && (0 < frames.length) )
            {
                _curFrame += framesToAdvance;
                _curFrame %= frames.length;
                updateFrameVisibility();

                timer %= timeBetweenFrames;
            }
        }

        /**
         * make sure only the current frame is visible
         */
        protected function updateFrameVisibility():void
        {
            for (var i:uint = 0; i < frames.length; ++i)
            {
                var displayObject:DisplayObject = frames[i];
                displayObject.visible = ( _visible && (curFrame == i) );
            }
        }
    }
}
