﻿package stego.widget
{
    import flash.display.Sprite;
    import flash.display.DisplayObjectContainer;
    import flash.geom.Rectangle;
    import stego.constants.Colors;
    import stego.core.GameObject;
    import stego.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PulsingText extends GameObject
    {
        protected var mPulseTextField:TextField;
        protected var mTextField:TextField;

        public function get hAlign():uint { return mTextField.hAlign; }
        public function set hAlign(value:uint):void
        {
            mTextField.hAlign = value;
            UpdatePulsePosition();
        }
        public function get vAlign():uint { return mTextField.vAlign; }
        public function set vAlign(value:uint):void
        {
            mTextField.vAlign = value;
            UpdatePulsePosition();
        }

        protected var mColor:Number = Colors.BLACK;

        private static const PULSE_MAX_SCALE:Number = 1.5;
        private static const PULSE_MAX_ALPHA:Number = 1.0;
        private static const PULSE_MIN_ALPHA:Number = 0.0;
        private static const PULSE_DURATION:Number = 0.75;

        // the pulse text seems to be too high, and I don't
        // have time to look into why
        private static const Y_FUDGE:Number = 0.0625;

        private var mPulseTimer:Number;

        public function PulsingText()
        {
            super();

            ResetDefaults();
        }

        public override function Init():void
        {
            super.Init();

            CreateGraphics();
            SetFontColor(mColor);
        }

        public function SetFontColor(color:Number):void
        {
            mColor = color;
            mPulseTextField.SetFontColor(mColor);
            mTextField.SetFontColor(mColor);
        }

        protected function CreateGraphics():void
        {
            var holder:Sprite = new Sprite();
            if (null == mTextField)
            {
                mTextField = new TextField();
                holder.addChild(mTextField);
            }

            if (null == mPulseTextField)
            {
                mPulseTextField = new TextField();
                holder.addChild(mPulseTextField);
            }

            this.addChild(holder);
        }

        public override function Beat(beat:Number):void
        {
            super.Beat(beat);

            mPulseTimer = 0;
        }

        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            mPulseTimer += elapsedTime;

            var t:Number = Math.min(1.0, mPulseTimer / PULSE_DURATION);
            mPulseTextField.alpha = MathUtil.Lerp(PULSE_MAX_ALPHA, PULSE_MIN_ALPHA, t);
            mPulseTextField.scaleX = MathUtil.Lerp( 1.0, PULSE_MAX_SCALE, t);
            mPulseTextField.scaleY = mPulseTextField.scaleX;
        }

        /**
         * center the textfield on some point.  CALL SET TEXT *FIRST*
         */
        public function CenterOn(x:Number, y:Number):void
        {
            this.x = x;
            this.y = y;
            mTextField.CenterOn(0, 0);
            UpdatePulsePosition();
        }

        /**
         * Change the displayed text
         */
        public function SetText(text:String):void
        {
            mTextField.SetText(text);

            mPulseTextField.SetText(text);
            UpdatePulsePosition();
        }

        /**
         * Change the displayed font size.
         */
        public function SetFontSize(desiredSize:Number):void
        {
            mTextField.SetFontSize(desiredSize);

            mPulseTextField.SetFontSize(desiredSize);
            UpdatePulsePosition();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        public override function Shutdown():void
        {
            ResetDefaults();

            mPulseTextField.alpha = PULSE_MAX_ALPHA;
            mPulseTextField.scaleX = 1.0;
            mPulseTextField.scaleY = 1.0;

            super.Shutdown();
        }

        /**
         * reset to defaults.  This thing will likely get re-used!
         */
        private function ResetDefaults():void
        {
            mPulseTimer = PULSE_DURATION;

            if (null != mPulseTextField)
            {
                mPulseTextField.ResetDefaults();
            }

            if (null != mTextField)
            {
                mTextField.ResetDefaults();
            }
        }

        /**
         * Change the font.  WILL SCREW UP ALIGNMENT.
         */
        public function SetFont(font:String):void
        {
            mTextField.SetFont(font);

            mPulseTextField.SetFont(font);
            UpdatePulsePosition();
        }

        /**
         * Update the pulse TextField's position to keep it centered on the real text
         */
        public function UpdatePulsePosition():void
        {
            var bounds:Rectangle = mTextField.getBounds(mTextField.parent);
            mPulseTextField.CenterOn(
                bounds.left + 0.5*bounds.width,
                bounds.top + 0.5*bounds.height + (Y_FUDGE * bounds.height ) );
        }
    }

}
